import { Directive, forwardRef, ElementRef, Renderer2, Input, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { FloatingMenuPlugin, FloatingMenuPluginKey } from '@tiptap/extension-floating-menu';
import { BubbleMenuPlugin, BubbleMenuPluginKey } from '@tiptap/extension-bubble-menu';

class EditorDirective {
    constructor(el, _renderer) {
        this.el = el;
        this._renderer = _renderer;
        this.outputFormat = 'html';
        this.onChange = () => { };
        this.onTouched = () => { };
        this.handleChange = ({ transaction }) => {
            if (!transaction.docChanged) {
                return;
            }
            if (this.outputFormat === 'html') {
                this.onChange(this.editor.getHTML());
                return;
            }
            this.onChange(this.editor.getJSON());
        };
    }
    // Writes a new value to the element.
    // This methods is called when programmatic changes from model to view are requested.
    writeValue(value) {
        if (!value) {
            return;
        }
        if (!this.outputFormat && typeof value === 'string') {
            this.outputFormat = 'html';
        }
        this.editor.chain().setContent(value, false).run();
    }
    // Registers a callback function that is called when the control's value changes in the UI.
    registerOnChange(fn) {
        this.onChange = fn;
    }
    // Registers a callback function that is called by the forms API on initialization to update the form model on blur.
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    // Called by the forms api to enable or disable the element
    setDisabledState(isDisabled) {
        this.editor.setEditable(!isDisabled);
        this._renderer.setProperty(this.el.nativeElement, 'disabled', isDisabled);
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        // take the inner contents and clear the block
        const innerHTML = this.el.nativeElement.innerHTML;
        this.el.nativeElement.innerHTML = '';
        // insert the editor in the dom
        this.el.nativeElement.appendChild(this.editor.options.element.firstChild);
        // update the options for the editor
        this.editor.setOptions({ element: this.el.nativeElement });
        // update content to the editor
        if (innerHTML) {
            this.editor.chain().setContent(innerHTML, false).run();
        }
        // register blur handler to update `touched` property
        this.editor.on('blur', () => {
            this.onTouched();
        });
        // register transaction handler to emit changes on update
        this.editor.on('transaction', this.handleChange);
    }
    ngOnDestroy() {
        this.editor.destroy();
    }
}
EditorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'tiptap, [tiptap], tiptap-editor, [tiptapEditor]',
                providers: [{
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => EditorDirective),
                        multi: true
                    }]
            },] }
];
EditorDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
EditorDirective.propDecorators = {
    editor: [{ type: Input }],
    outputFormat: [{ type: Input }]
};

class FloatingMenuDirective {
    constructor(_el) {
        this._el = _el;
        this.tippyOptions = {};
    }
    ngOnInit() {
        this.editor.registerPlugin(FloatingMenuPlugin({
            editor: this.editor,
            element: this._el.nativeElement,
            tippyOptions: this.tippyOptions
        }));
    }
    ngOnDestroy() {
        this.editor.unregisterPlugin(FloatingMenuPluginKey);
    }
}
FloatingMenuDirective.decorators = [
    { type: Directive, args: [{
                selector: 'tiptap-floting-menu, [tiptapFloatingMenu]'
            },] }
];
FloatingMenuDirective.ctorParameters = () => [
    { type: ElementRef }
];
FloatingMenuDirective.propDecorators = {
    editor: [{ type: Input }],
    tippyOptions: [{ type: Input }]
};

class BubbleMenuDirective {
    constructor(_el) {
        this._el = _el;
        this.tippyOptions = {};
    }
    ngOnInit() {
        this.editor.registerPlugin(BubbleMenuPlugin({
            editor: this.editor,
            element: this._el.nativeElement,
            tippyOptions: this.tippyOptions
        }));
    }
    ngOnDestroy() {
        this.editor.unregisterPlugin(BubbleMenuPluginKey);
    }
}
BubbleMenuDirective.decorators = [
    { type: Directive, args: [{
                selector: 'tiptap-bubble-menu, [tiptapBubbleMenu]'
            },] }
];
BubbleMenuDirective.ctorParameters = () => [
    { type: ElementRef }
];
BubbleMenuDirective.propDecorators = {
    editor: [{ type: Input }],
    tippyOptions: [{ type: Input }]
};

class NgxTiptapModule {
}
NgxTiptapModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    EditorDirective,
                    FloatingMenuDirective,
                    BubbleMenuDirective,
                ],
                exports: [
                    EditorDirective,
                    FloatingMenuDirective,
                    BubbleMenuDirective
                ]
            },] }
];

/*
 * Public API Surface of ngx-tiptap
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BubbleMenuDirective, EditorDirective, FloatingMenuDirective, NgxTiptapModule };
//# sourceMappingURL=ngx-tiptap.js.map
